<?php
defined('_JEXEC') or die('Direct access is not allowed.');

if (!class_exists('vmPSPlugin'))
    require(JPATH_VM_PLUGINS . DS . 'vmpsplugin.php');
if (!class_exists('ForteEcomHelper'))
    require(__DIR__ . DS . 'forteecom' . DS . 'classes' . DS . 'ForteEcomHelper.php');

class plgVmPaymentForteecom extends vmPSPlugin
{

    function __construct(&$subject, $config)
    {
        parent::__construct($subject, $config);

        $this->requeryCount = 0;

        $this->_loggable  = true;
        $this->_tablepkey = 'id';
        $this->_tableId   = 'id';

        $this->tableFields = array_keys($this->getTableSQLFields());

        $varsToPush = array(
            'test_mode'   => array(
                1,
                'int'
            ),
            'merchant_id' => array(
                '',
                'char'
            ),
            'pre_auth'    => array(
                0,
                'int'
            ),
            'description' => array(
                '',
                'char'
            ),
        );

        $this->setConfigParameterable($this->_configTableFieldName, $varsToPush);
    }

    function getTableSQLFields()
    {
        return array(
            'id'                          => 'tinyint(1) unsigned NOT NULL AUTO_INCREMENT',
            'virtuemart_order_id'         => 'int(11) UNSIGNED DEFAULT NULL',
            'forteecom_order_id'          => 'char(255) NOT NULL',
            'forteecom_order_status'      => 'char(255) NOT NULL',
            'forteecom_session_id'        => 'char(255) NOT NULL',
            'forteecom_merchant_id'       => 'char(255) NOT NULL',
            'order_number'                => 'char(32) DEFAULT NULL',
            'virtuemart_paymentmethod_id' => 'mediumint(1) UNSIGNED DEFAULT NULL',
            'payment_name'                => 'char(255) NOT NULL DEFAULT \'\' ',
            'payment_order_total'         => 'decimal(15,5) NOT NULL DEFAULT \'0.00000\' ',
            'payment_refund_total'        => 'decimal(15,5) NOT NULL DEFAULT \'0.00000\' ',
            'payment_currency'            => 'char(3) ',
            'cost_per_transaction'        => 'decimal(10,2) DEFAULT NULL ',
            'cost_percent_total'          => 'decimal(10,2) DEFAULT NULL ',
        );
    }

    public function getVmPluginCreateTableSQL()
    {
        return $this->createTableSQL('Payment ForteEcom Table');
    }

    /**
     * @param VirtueMartCart $cart
     * @param                $order
     *
     * @return bool|null|void
     *
     * @since version
     */
    function plgVmConfirmedOrder($cart, $order)
    {
        $order_info = $order['details']['BT'];
        if (!($method = $this->getVmPluginMethod($order_info->virtuemart_paymentmethod_id))) {
            return null;
        }

        if (!$this->selectedThisElement($method->payment_element)) {
            return false;
        }

        if (!class_exists('VirtueMartModelOrders'))
            require(JPATH_VM_ADMINISTRATOR . DS . 'models' . DS . 'orders.php');

        if (!class_exists('VirtueMartModelCurrency'))
            require(JPATH_VM_ADMINISTRATOR . DS . 'models' . DS . 'currency.php');

        $forteecom_settings  = $this->getForteEcomSettings($order_info->virtuemart_paymentmethod_id);
        $payment_currency_id = shopFunctions::getCurrencyIDByName('KZT');
        $this->getPaymentCurrency($method);
        $totalInPaymentCurrency = vmPSPlugin::getAmountInCurrency($order_info->order_total, $payment_currency_id);
        $total                  = $totalInPaymentCurrency['value'];

        $order_description = JFactory::getConfig()->get('sitename') . ' - Order: ' . $order_info->order_number;

        $init_result = ForteEcomHelper::init_payment(
            $forteecom_settings['merchant_id'],
            $forteecom_settings['pre_auth'],
            $total,
            $order_info->order_number,
            $order_description,
            'KZT',
            $this
        );

        if (!empty($init_result['error_message'])) {
            $this->debugLog($init_result['error_message'], "ForteEcom: init", 'info');
            $html = "<p style='color:red; font-size: 20px; font-weight: bold'>" . vmText::_('VMPAYMENT_FORTEECOM_COULD_NOT_INIT') . "</p>";
        } else {
            $this->debugLog('payment inited with id ' . $init_result['_forteecom_order_id'], "ForteEcom: init", 'info');

            $init_information  = [
                'forteecom_order_id'    => $init_result['_forteecom_order_id'],
                'forteecom_session_id'  => $init_result['_forteecom_session_id'],
                'forteecom_merchant_id' => $forteecom_settings['merchant_id'],
            ];
            $order_information = [
                'number'      => $order_info->order_number,
                'description' => $order_description,
                'total'       => $total
            ];
            $db_values         = array_merge($init_information, [
                'order_number'                => $order_info->order_number,
                'payment_name'                => $this->renderPluginName($method),
                'forteecom_order_status'      => ForteEcomHelper::STATUS_INIT,
                'virtuemart_paymentmethod_id' => $order_info->virtuemart_paymentmethod_id,
                'payment_currency'            => $payment_currency_id,
                'payment_order_total'         => $total
            ]);
            $this->storePSPluginInternalData($db_values);

            $redirect_query = http_build_query(array(
                'option' => 'com_virtuemart',
                'view'   => 'pluginresponse',
                'task'   => 'pluginresponsereceived',
                'on'     => $order_info->order_number,
                'pm'     => $order_info->virtuemart_paymentmethod_id,
                'Itemid' => vRequest::getInt('Itemid'),
            ));

            $notify_query = http_build_query(array(
                'option' => 'com_virtuemart',
                'view'   => 'pluginresponse',
                'task'   => 'pluginnotification',
                'format' => 'raw',
                'on'     => $order_info->order_number,
                'pm'     => $order_info->virtuemart_paymentmethod_id,
                'Itemid' => vRequest::getInt('Itemid'),
            ));

            $html = $this->renderByLayout('order_checkout_form', [
                'init_info'    => $init_information,
                'order_info'   => $order_information,
                'stored_cards' => $this->getStoredCards(),
                'urls'         => [
                    'redirect' => JURI::root() . 'index.php?' . $redirect_query,
                    'notify'   => JURI::root() . 'index.php?' . $notify_query,
                ],
                'script'       => ForteEcomHelper::get_script(),
            ]);
            $cart->_confirmDone = false;
            $cart->_dataValidated = false;
            $cart->setCartIntoSession();
        }

        vRequest::setVar('html', $html);
    }

    function getForteEcomSettings($payment_method_id)
    {
        $forteecom_settings = $this->getPluginMethod($payment_method_id);

        define('FORTEECOM_DEMO_ENABLED', $forteecom_settings->test_mode ? 1 : 0);

        return array(
            'merchant_id' => $forteecom_settings->merchant_id,
            'pre_auth'    => ($forteecom_settings->pre_auth) ? true : false,
            'test_mode'   => ($forteecom_settings->test_mode) ? true : false
        );
    }

    /**
     * @param $html
     *
     * @return bool|string|void|null
     * @throws Exception
     */
    function plgVmOnPaymentResponseReceived(&$html)
    {
        if (!class_exists('VirtueMartCart')) {
            require(VMPATH_SITE . DS . 'helpers' . DS . 'cart.php');
        }
        if (!class_exists('shopFunctionsF')) {
            require(VMPATH_SITE . DS . 'helpers' . DS . 'shopfunctionsf.php');
        }
        if (!class_exists('VirtueMartModelOrders')) {
            require(VMPATH_ADMIN . DS . 'models' . DS . 'orders.php');
        }

        VmConfig::loadJLang('com_virtuemart_orders', true);
        $virtuemart_paymentmethod_id = vRequest::getInt('pm', 0);

        $order_number = vRequest::getString('on', 0);
        if (!($method = $this->getVmPluginMethod($virtuemart_paymentmethod_id))) {
            return null;
        }

        if (!$this->selectedThisElement($method->payment_element)) {
            return null;
        }

        if (!($virtuemart_order_id = VirtueMartModelOrders::getOrderIdByOrderNumber($order_number))) {
            return null;
        }

        if (!($paymentTable = $this->getDataByOrderId($virtuemart_order_id))) {
            return '';
        }

        VmConfig::loadJLang('com_virtuemart');
        $orderModel              = VmModel::getModel('orders');
        $order                   = $orderModel->getOrder($virtuemart_order_id);
        $_forteecom_order_id     = $paymentTable->forteecom_order_id;
        $_forteecom_order_status = $paymentTable->forteecom_order_status;
        $_forteecom_session_id   = $paymentTable->forteecom_session_id;
        $_forteecom_merchant_id  = $paymentTable->forteecom_merchant_id;
        $order_description       = JFactory::getConfig()->get('sitename') . ' - Order: ' . $order_number;

        $this->getForteEcomSettings($order['details']['BT']->virtuemart_paymentmethod_id);

        $query_args    = array(
            'option'       => 'com_virtuemart',
            'view'         => 'orders',
            'layout'       => 'details',
            'order_number' => $order_number,
        );
        $button_text   = vmText::_('COM_VIRTUEMART_ORDER_VIEW_ORDER');
        $error_message = '';
        $status_char   = $order['details']['BT']->order_status;

        if ($order['details']['BT']->order_status == 'P') {
            $request = vRequest::getPost();
            if ($request['forteecom_payment_method'] == 'new') {
                $status_result = ForteEcomHelper::get_payment_status(
                    $_forteecom_merchant_id,
                    $_forteecom_order_id,
                    $_forteecom_session_id,
                    $this
                );
            } else {

                $card_id     = $request['forteecom_payment_method'];
                $stored_card = $this->getStoredCardById($card_id);
                if (empty($stored_card)) {
                    // This should never happen
                    return;
                }
                $status_result = ForteEcomHelper::purchase_by_token(
                    $_forteecom_merchant_id,
                    $_forteecom_order_id,
                    $_forteecom_session_id,
                    $paymentTable->payment_order_total,
                    $order_number,
                    $order_description,
                    398,
                    $stored_card,
                    $this
                );

            }

            if (!empty($status_result['error_message'])) {
                $error_message = $status_result['error_message'];
                $this->debugLog('Could not get status for order #' . $order_number, 'ForteEcom: status', 'error');
            }  else {
                ForteEcomHelper::process_payment_status($order, $_forteecom_order_id, $status_result['_forteecom_order_status'], $_forteecom_order_status, $this, $button_text, $query_args);
                $internal_data                         = $this->getPluginInternalData($paymentTable->id);
                $internal_data->forteecom_order_status = $status_result['_forteecom_order_status'];
                $internal_data->store();
            }

            $order['customer_notified'] = 1;
            $status_char                = $order['order_status'];
            $orderModel->updateStatusForOneOrder($order['details']['BT']->virtuemart_order_id, $order, true);

        }

        $payment_name = $this->renderPluginName($method);

        $html = '<table>';
        $html .= $this->getHtmlRow(vmText::_('VMPAYMENT_FORTEECOM_PAYMENT_NAME'), $payment_name);
        $html .= $this->getHtmlRow(vmText::_('VMPAYMENT_FORTEECOM_ORDER_NUMBER'), $order_number);
        $html .= $this->getHtmlRow(vmText::_('VMPAYMENT_FORTEECOM_STATUS'), vmText::_('VMPAYMENT_FORTEECOM_ORDERSTATUS_' . $status_char));
        $html .= $this->getHtmlRow(vmText::_('VMPAYMENT_FORTEECOM_TOTAL_AMOUNT'), $paymentTable->payment_order_total);
        $html .= '</table>';
        $html .= '<span class="error_message">' . $error_message . '</span>';
        $html .= '<br/>';
        $html .= '<a href="' . JRoute::_('index.php?' . http_build_query($query_args), false) . '" class="vm-button-correct">' . $button_text . '</a>';

        return false;
    }

    /**
     * Process webhook notifications
     *
     * @return bool|string|null
     */
    public function plgVmOnPaymentNotification()
    {
        if (!class_exists('VirtueMartCart')) {
            require(VMPATH_SITE . DS . 'helpers' . DS . 'cart.php');
        }
        if (!class_exists('shopFunctionsF')) {
            require(VMPATH_SITE . DS . 'helpers' . DS . 'shopfunctionsf.php');
        }
        if (!class_exists('VirtueMartModelOrders')) {
            require(VMPATH_ADMIN . DS . 'models' . DS . 'orders.php');
        }

        $virtuemart_paymentmethod_id = vRequest::getInt('pm', 0);


        if (!($method = $this->getVmPluginMethod($virtuemart_paymentmethod_id))) {
            return null;
        }

        if (!$this->selectedThisElement($method->payment_element)) {
            return null;
        }

        $request =  json_decode( file_get_contents( "php://input" ), true );

        switch ($request['type']) {
            case 'PaymentComplete':
                $order_number = vRequest::getString('on', 0);
                if (!($virtuemart_order_id = VirtueMartModelOrders::getOrderIdByOrderNumber($order_number))) {
                    return null;
                }

                if (!($paymentTable = $this->getDataByOrderId($virtuemart_order_id))) {
                    return '';
                }

                VmConfig::loadJLang('com_virtuemart');
                $orderModel              = VmModel::getModel('orders');
                $order                   = $orderModel->getOrder($virtuemart_order_id);
                $_forteecom_order_id     = $paymentTable->forteecom_order_id;
                $_forteecom_order_status = $paymentTable->forteecom_order_status;
                $_forteecom_session_id   = $paymentTable->forteecom_session_id;
                $_forteecom_merchant_id  = $paymentTable->forteecom_merchant_id;

                $this->getForteEcomSettings($order['details']['BT']->virtuemart_paymentmethod_id);

                $status_result = ForteEcomHelper::get_payment_status(
                    $_forteecom_merchant_id,
                    $_forteecom_order_id,
                    $_forteecom_session_id,
                    $this
                );

                ForteEcomHelper::process_payment_status($order, $_forteecom_order_id, $status_result['_forteecom_order_status'], $_forteecom_order_status);
                $orderModel->updateStatusForOneOrder($virtuemart_order_id, $order, true);
                $internal_data                         = $this->getPluginInternalData($paymentTable->id);
                $internal_data->forteecom_order_status = $status_result['_forteecom_order_status'];
                $internal_data->store();
                break;
            case 'CardSaved':
                $this->storeCreditCard(
                    $request['data']['card_token'],
                    $request['data']['card_last4'],
                    $request['data']['card_expiry_year'],
                    $request['data']['card_expiry_month'],
                    $request['data']['card_type']
                );
                break;
            case 'PaymentRefunded':
                $order_number = vRequest::getString('on', 0);
                if (!($virtuemart_order_id = VirtueMartModelOrders::getOrderIdByOrderNumber($order_number))) {
                    return null;
                }

                if (!($paymentTable = $this->getDataByOrderId($virtuemart_order_id))) {
                    return '';
                }

                VmConfig::loadJLang('com_virtuemart');
                $orderModel              = VmModel::getModel('orders');
                $order                   = $orderModel->getOrder($virtuemart_order_id);
                $_forteecom_order_id     = $paymentTable->forteecom_order_id;
                $_forteecom_session_id   = $paymentTable->forteecom_session_id;
                $_forteecom_merchant_id  = $paymentTable->forteecom_merchant_id;
                $_forteecom_order_status = $paymentTable->forteecom_order_status;

                $this->getForteEcomSettings($order['details']['BT']->virtuemart_paymentmethod_id);

                $refund_result = ForteEcomHelper::refund_payment(
                    $_forteecom_merchant_id,
                    $_forteecom_order_id,
                    $_forteecom_session_id,
                    $request['data']['amount'],
                    $request['data']['currency_code'],
                    $this
                );

                $success = false;
                if (empty( $refund_result['error_message'] )) {
                    $success = true;
                    ForteEcomHelper::process_payment_status($order, $_forteecom_order_id, ForteEcomHelper::STATUS_REFUND, $_forteecom_order_status);
                    $orderModel->updateStatusForOneOrder($virtuemart_order_id, $order, true);
                    $internal_data                         = $this->getPluginInternalData($paymentTable->id);
                    $internal_data->forteecom_order_status = ForteEcomHelper::STATUS_REFUND;
                    $internal_data->payment_refund_total   = $paymentTable->payment_refund_total + $request['data']['amount'];
                    $internal_data->store();
                }
                

                $response = [
                    'success'       => $success,
                    'error_message' => $refund_result['error_message']
                ];

                echo json_encode($response);
                break;
            case 'PaymentRetried':
                $order_number = vRequest::getString('on', 0);
                if (!($virtuemart_order_id = VirtueMartModelOrders::getOrderIdByOrderNumber($order_number))) {
                    return null;
                }

                if (!($paymentTable = $this->getDataByOrderId($virtuemart_order_id))) {
                    return '';
                }

                VmConfig::loadJLang('com_virtuemart');
                $orderModel              = VmModel::getModel('orders');
                $order                   = $orderModel->getOrder($virtuemart_order_id);
                $order_info              = $order['details']['BT'];
                $_forteecom_order_id     = $paymentTable->forteecom_order_id;
                $_forteecom_order_status = $paymentTable->forteecom_order_status;
                $_forteecom_session_id   = $paymentTable->forteecom_session_id;
                $_forteecom_merchant_id  = $paymentTable->forteecom_merchant_id;
                $forteecom_settings      = $this->getForteEcomSettings($order['details']['BT']->virtuemart_paymentmethod_id);

                $status_result = ForteEcomHelper::get_payment_status(
                    $_forteecom_merchant_id,
                    $_forteecom_order_id,
                    $_forteecom_session_id,
                    $this
                );

                if (in_array($status_result['_forteecom_order_status'], [ForteEcomHelper::STATUS_DECLINE, ForteEcomHelper::STATUS_EXPIRE, ForteEcomHelper::STATUS_CANCEL, ForteEcomHelper::STATUS_ERROR])) {

                    $forteecom_settings     = $this->getForteEcomSettings($order_info->virtuemart_paymentmethod_id);
                    $payment_currency_id    = shopFunctions::getCurrencyIDByName('KZT');
                    $this->getPaymentCurrency($method);
                    $totalInPaymentCurrency = vmPSPlugin::getAmountInCurrency($order_info->order_total, $payment_currency_id);
                    $total                  = $totalInPaymentCurrency['value'];
                    $order_description      = JFactory::getConfig()->get('sitename') . ' - Order: ' . $order_info->order_number;

                    $init_result = ForteEcomHelper::init_payment(
                        $forteecom_settings['merchant_id'],
                        $forteecom_settings['pre_auth'],
                        $total,
                        $order_info->order_number,
                        $order_description,
                        'KZT',
                        $this
                    );
                    $init_information  = [
                        'forteecom_order_id'    => $init_result['_forteecom_order_id'],
                        'forteecom_session_id'  => $init_result['_forteecom_session_id'],
                        'forteecom_merchant_id' => $forteecom_settings['merchant_id'],
                    ];
                    $db_values         = array_merge($init_information, [
                        'order_number'                => $order_info->order_number,
                        'payment_name'                => $this->renderPluginName($method),
                        'forteecom_order_status'      => ForteEcomHelper::STATUS_INIT,
                        'virtuemart_paymentmethod_id' => $order_info->virtuemart_paymentmethod_id,
                        'payment_currency'            => $payment_currency_id,
                        'payment_order_total'         => $total
                    ]);
                    $this->storePSPluginInternalData($db_values);
                    ForteEcomHelper::process_payment_status($order, $_forteecom_order_id, ForteEcomHelper::STATUS_INIT, $_forteecom_order_status);
                    $orderModel->updateStatusForOneOrder($virtuemart_order_id, $order, true);
                    $internal_data                         = $this->getPluginInternalData($paymentTable->id);
                    $internal_data->forteecom_order_status = $status_result['_forteecom_order_status'];
                    $internal_data->store();

                    $response = [
                        'sessionId' => $init_result['_forteecom_session_id'],
                        'orderId'   => $init_result['_forteecom_order_id']
                    ];
                } else {
                    $response = [
                        'sessionId' => $_forteecom_session_id,
                        'orderId'   => $_forteecom_order_id
                    ];
                }

                
                
                echo json_encode($response);
                break;
        }

        return true;
    }

    function plgVmOnUpdateOrderPayment($virtuemart_order, $old_status)
    {
        $virtuemart_order_id = $virtuemart_order->virtuemart_order_id;
        if (!($method = $this->getVmPluginMethod($virtuemart_order->virtuemart_paymentmethod_id))) {
            return;
        }
        if (!$this->selectedThisElement($method->payment_element)) {
            return;
        }
        if (!($paymentTable = $this->getDataByOrderId($virtuemart_order_id))) {
            return;
        }

        $orderModel = VmModel::getModel('orders');
        $order      = $orderModel->getOrder($virtuemart_order_id);
        $this->getForteEcomSettings($order['details']['BT']->virtuemart_paymentmethod_id);
        if ($paymentTable->forteecom_order_status == ForteEcomHelper::STATUS_ON_HOLD && in_array($virtuemart_order->order_status, array('F', 'S', 'U'))) {

            $complete_result = ForteEcomHelper::complete_payment(
                $paymentTable->forteecom_merchant_id,
                $paymentTable->forteecom_order_id,
                $paymentTable->forteecom_session_id,
                $this
            );

            if (empty($complete_result['error_message'])) {
                $internal_data                         = $this->getPluginInternalData($paymentTable->id);
                $internal_data->forteecom_order_status = ForteEcomHelper::STATUS_SUCCESS;
                $internal_data->store();
            } else {
                $order['order_status'] = 'C';
                $orderModel->updateStatusForOneOrder($virtuemart_order_id, $order, true);
                $message = 'Error occurred while changing payment status from on-hold to complete';
                $this->debugLog($message, 'Forteecom COMPLETE', 'error', true);
                try {
                    JFactory::getApplication()->enqueueMessage($message, 'error');
                } catch (Exception $e) {
                }
            }
        }
    }

    function plgVmOnShowOrderBEPayment($virtuemart_order_id, $payment_method_id)
    {

        if (!$this->selectedThisByMethodId($payment_method_id)) {
            return NULL; // Another method was selected, do nothing
        }
        if (!($this->_currentMethod = $this->getVmPluginMethod($payment_method_id))) {
            return FALSE;
        }

        $paymentTable = $this->getDataByOrderId($virtuemart_order_id);

        VmConfig::loadJLang('com_virtuemart');
        $orderModel = VmModel::getModel('orders');
        $order      = $orderModel->getOrder($virtuemart_order_id);
        $order_info = $order['details']['BT'];

        $currency_code = shopFunctions::getCurrencyByID($paymentTable->payment_currency, 'currency_code_3');
        
        $notify_query = http_build_query(array(
            'option' => 'com_virtuemart',
            'view'   => 'pluginresponse',
            'task'   => 'pluginnotification',
            'format' => 'raw',
            'on'     => $order_info->order_number,
            'pm'     => $payment_method_id,
            'Itemid' => vRequest::getInt('virtuemart_order_id'),
        ));
        $notify_url = JURI::root() . 'index.php?' . $notify_query;
        vmJsApi::css('main', 'plugins/vmpayment/forteecom/forteecom/assets/css');
        vmJsApi::addJScript('/plugins/vmpayment/forteecom/forteecom/assets/js/main.js');

        $html = '<table class="adminlist table" >';
        $html .= $this->getHtmlHeaderBE();
        $html .= $this->getHtmlRowBE('COM_VIRTUEMART_PAYMENT_NAME', $paymentTable->payment_name);
        $html .= $this->getHtmlRowBE(
            'VMPAYMENT_FORTEECOM_ORDERED_TOTAL', 
            $paymentTable->payment_order_total . " " . $currency_code
        );
        $html .= $this->getHtmlRow(
            'VMPAYMENT_FORTEECOM_REFUNDED_TOTAL', 
            $paymentTable->payment_refund_total . " " . $currency_code,
            'class="text-error"'
        );
        $html .= $this->getHtmlRowBE(
            'VMPAYMENT_FORTEECOM_TOTAL_AMOUNT', 
            $paymentTable->payment_order_total - $paymentTable->payment_refund_total . " " . $currency_code
        );
        $html .= $this->getHtmlRowBE(
            'VMPAYMENT_FORTEECOM_CREATE_NEW_REFUND', 
            '<input type="number" id="refund_amount" step="0.00001" value="0" /> ' . $currency_code . '<br>
            <button onclick="jQuery.forteecom_refund(
                `' . $notify_url . '`, 
                jQuery(`#refund_amount`).val(),
                `' . $currency_code . '`,
                `' . vmText::_('VMPAYMENT_FORTEECOM_REFUND_CONFIRM_TEXT') . '`
            );">' . vmText::_('VMPAYMENT_FORTEECOM_PROCESS_REFUND') . '</button>'
        );
        $html .= '</table>';
        $html .= '<div class="forteecom_overlay"><div class="forteecom_outer"><p>' . vmText::_('VMPAYMENT_FORTEECOM_TIP_DO_NOT_CLOSE_WINDOW') . '</p></div></div>';

        return $html;
    }

    public function plgVmOnForteEcomGetSettings(&$settings)
    {
        $vendorId = 1;
        if ($this->getPluginMethods($vendorId) === 0) {
            return;
        }
        foreach ($this->methods as $method) {
            $settings = $this->getForteEcomSettings($method->virtuemart_paymentmethod_id);
        }
    }

    function getStoredCards($virtuemart_user_id = null)
    {
        JLoader::import('joomla.plugin.helper');
        JPluginHelper::importPlugin('vmuserfield');
        $app = JFactory::getApplication();

        if (!isset($virtuemart_user_id)) {
            $virtuemart_user_id = JFactory::getUser()->get('id');
        }

        $stored_cards = "";
        $app->triggerEvent('plgVmOnForteEcomGetStoredCards', array($virtuemart_user_id, &$stored_cards));

        return $stored_cards;
    }

    public function plgVmOnForteEcomGetScript(&$script)
    {
        $app = JFactory::getApplication();
        $script = ForteEcomHelper::get_script();
    }

    /**
     * @param      $card_id
     * @param null $virtuemart_user_id
     *
     * @return mixed|string
     * @throws Exception
     */
    function getStoredCardById($card_id, $virtuemart_user_id = null)
    {
        JLoader::import('joomla.plugin.helper');
        JPluginHelper::importPlugin('vmuserfield');
        $app = JFactory::getApplication();

        if (!isset($virtuemart_user_id)) {
            $virtuemart_user_id = JFactory::getUser()->get('id');
        }

        $stored_cards = "";
        $app->triggerEvent('plgVmOnForteEcomGetStoredCards', array($virtuemart_user_id, &$stored_cards, $card_id));

        if (!empty($stored_cards)) return $stored_cards[0];
        return $stored_cards;
    }

    function plgVmOnUserPaymentCancel()
    {
        return true;
    }

    function plgVmOnStoreInstallPaymentPluginTable($jplugin_id)
    {
        return $this->onStoreInstallPluginTable($jplugin_id);
    }

    public function plgVmOnSelectCheckPayment(VirtueMartCart $cart)
    {
        return $this->OnSelectCheck($cart);
    }

    public function plgVmDisplayListFEPayment(VirtueMartCart $cart, $selected = 0, &$htmlIn)
    {
        return $this->displayListFE($cart, $selected, $htmlIn);
    }

    public function plgVmonSelectedCalculatePricePayment(VirtueMartCart $cart, array &$cart_prices, &$cart_prices_name)
    {
        return $this->onSelectedCalculatePrice($cart, $cart_prices, $cart_prices_name);
    }

    function plgVmOnCheckAutomaticSelectedPayment(VirtueMartCart $cart, array $cart_prices = array(), &$paymentCounter)
    {
        return $this->onCheckAutomaticSelected($cart, $cart_prices, $paymentCounter);
    }

    /**
     * Show additional info for Order in FrontEnd
     */
    public function plgVmOnShowOrderFEPayment($virtuemart_order_id, $virtuemart_paymentmethod_id, &$payment_name)
    {
        $this->onShowOrderFE($virtuemart_order_id, $virtuemart_paymentmethod_id, $payment_name);
    }

    function plgVmonShowOrderPrintPayment($order_number, $method_id)
    {
        return $this->onShowOrderPrint($order_number, $method_id);
    }

    function plgVmDeclarePluginParamsPaymentVM3(&$data)
    {
        return $this->declarePluginParams('payment', $data);
    }

    function plgVmSetOnTablePluginParamsPayment($name, $id, &$table)
    {
        return $this->setOnTablePluginParams($name, $id, $table);
    }

    function plgVmgetPaymentCurrency($virtuemart_paymentmethod_id, &$paymentCurrencyId)
    {

        if (!($this->_currentMethod = $this->getVmPluginMethod($virtuemart_paymentmethod_id))) {
            return NULL; // Another method was selected, do nothing
        }
        if (!$this->selectedThisElement($this->_currentMethod->payment_element)) {
            return FALSE;
        }
        $this->_currentMethod->payment_currency = 'KZT';

        if (!class_exists('VirtueMartModelVendor')) {
            require(JPATH_VM_ADMINISTRATOR . DS . 'models' . DS . 'vendor.php');
        }
        $db = JFactory::getDBO();
        $q  = 'SELECT   `virtuemart_currency_id` FROM `#__virtuemart_currencies` WHERE `currency_code_3`= "' . $this->_currentMethod->payment_currency . '"';
        $db->setQuery($q);
        $paymentCurrencyId = $db->loadResult();
    }

    protected function checkConditions($cart, $method, $cart_prices)
    {
        if (shopFunctions::getCurrencyByID($cart->pricesCurrency, 'currency_code_3') != 'KZT') return false;
        return true;
    }

    private function storeCreditCard($token, $last4, $expiration_year, $expiration_month, $type)
    {
        JLoader::import('joomla.plugin.helper');
        JPluginHelper::importPlugin('vmuserfield');
        $app                = JFactory::getApplication();
        $virtuemart_user_id = JFactory::getUser()->get('id');

        $db_values = [
            'virtuemart_user_id'    => $virtuemart_user_id,
            'card_token'            => $token,
            'card_last4'            => $last4,
            'card_expiration_year'  => $expiration_year,
            'card_expiration_month' => $expiration_month,
            'card_type'             => $type,
        ];

        $app->triggerEvent('plgVmOnForteEcomStoreCreditCard', [$db_values]);
    }

}